@@ -12,7 +12,7 @@ __author__ = "Shyue Ping Ong"
 __copyright__ = "Copyright 2012, The Materials Project"
 __version__ = "0.1"
 __maintainer__ = "Shyue Ping Ong"
-__email__ = "shyue@mit.edu"
+__email__ = "ongsp@ucsd.edu"
 __date__ = "Jun 2, 2012"
 
 
