@@ -114,9 +114,12 @@ class Custodian(object):
                 scratch partition has much faster IO. To use this, set
                 scratch_dir=root of directory you want to use for runs.
                 There is no need to provide unique directory names; we will
-                use python's tempfile creation mechanisms. If this is
-                None (the default), the run is performed in the current
-                working directory.
+                use python's tempfile creation mechanisms. A symbolic link
+                is created during the course of the run in the working
+                directory called "scratch_link" as users may want to
+                sometimes check the output during the course of a run. If
+                this is None (the default), the run is performed in the
+                current working directory.
             gzipped_output:
                 Whether to gzip the final output to save space. Defaults to
                 False.
@@ -178,130 +181,122 @@ class Custodian(object):
         """
         cwd = os.getcwd()
 
-        if self.scratch_dir is not None:
-            tempdir = tempfile.mkdtemp(dir=self.scratch_dir)
-            for f in os.listdir("."):
-                shutil.copy(f, tempdir)
-            os.chdir(tempdir)
-            logging.info("Using scratch directory {}.".format(tempdir))
+        with ScratchDir(self.scratch_dir):
 
-        total_errors = 0
-        unrecoverable = False
-        start = datetime.datetime.now()
-        logging.info("Run started at {}.".format(start))
+            total_errors = 0
+            unrecoverable = False
+            start = datetime.datetime.now()
+            logging.info("Run started at {}.".format(start))
 
-        if self.checkpoint:
-            restart, run_log = Custodian._load_checkpoint(cwd)
-        else:
-            restart, run_log = -1, []
-
-        for i, job in enumerate(self.jobs):
-            if i <= restart:
-                #Skip all jobs until the restart point.
-                continue
-            run_log.append({"job": job.to_dict, "corrections": []})
-            for attempt in xrange(self.max_errors):
-                logging.info(
-                    "Starting job no. {} ({}) attempt no. {}. Errors thus far"
-                    " = {}.".format(i + 1, job.name, attempt + 1,
-                                    total_errors))
-
-                # If this is the start of the job, do the setup.
-                if not run_log[-1]["corrections"]:
-                    job.setup()
-
-                p = job.run()
-                # Check for errors using the error handlers and perform
-                # corrections.
-                has_error = False
-
-                # While the job is running, we use the handlers that are
-                # monitors to monitor the job.
-                if isinstance(p, subprocess.Popen):
-                    if self.monitors:
-                        n = 0
-                        while True:
-                            n += 1
-                            time.sleep(self.polling_time_step)
-                            if p.poll() is not None:
-                                break
-                            if n % self.monitor_freq == 0:
-                                corrections = _do_check(
-                                    self.monitors, terminate_func=p.terminate,
-                                    skip_over_errors=self.skip_over_errors)
-                                if len(corrections) > 0:
-                                    has_error = True
-                                    total_errors += len(corrections)
-                                    run_log[-1]["corrections"].extend(
-                                        corrections)
-                    else:
-                        p.wait()
-
-                # Check for errors again, since in some cases non-monitor
-                # handlers fix the problems detected by monitors
-                # if an error has been found, not all handlers need to run
-                if has_error:
-                    remaining_handlers = filter(lambda x: not x.is_monitor,
-                                                self.handlers)
-                else:
-                    remaining_handlers = self.handlers
-
-                corrections = _do_check(
-                    remaining_handlers,
-                    skip_over_errors=self.skip_over_errors)
-                if len(corrections) > 0:
-                    has_error = True
-                    total_errors += len(corrections)
-                    run_log[-1]["corrections"].extend(corrections)
-
-                #Log the corrections to a json file.
-                with open(Custodian.LOG_FILE, "w") as f:
-                    logging.info("Logging to {}...".format(Custodian.LOG_FILE))
-                    json.dump(run_log, f, indent=4)
-
-                # If there are no errors detected, perform postprocessing and
-                # exit.
-                if not has_error:
-                    job.postprocess()
-                    break
-                elif run_log[-1]["corrections"][-1]["actions"] is None:
-                    # Check if there has been an unrecoverable error.
-                    logging.info("Unrecoverable error.")
-                    unrecoverable = True
-                    break
-                elif total_errors >= self.max_errors:
-                    logging.info("Max errors reached.")
-                    break
-
-            if unrecoverable or total_errors >= self.max_errors:
-                break
-
-            # Checkpoint after each job so that we can recover from last
-            # point and remove old checkpoints
             if self.checkpoint:
-                Custodian._save_checkpoint(cwd, i)
+                restart, run_log = Custodian._load_checkpoint(cwd)
+            else:
+                restart, run_log = -1, []
 
-        end = datetime.datetime.now()
-        logging.info("Run ended at {}.".format(end))
-        run_time = end - start
+            for i, job in enumerate(self.jobs):
+                if i <= restart:
+                    #Skip all jobs until the restart point.
+                    continue
+                run_log.append({"job": job.to_dict, "corrections": []})
+                for attempt in xrange(self.max_errors):
+                    logging.info(
+                        "Starting job no. {} ({}) attempt no. {}. Errors "
+                        "thus far = {}.".format(
+                            i + 1, job.name, attempt + 1, total_errors))
 
-        logging.info("Run completed. Total time taken = {}.".format(run_time))
+                    # If this is the start of the job, do the setup.
+                    if not run_log[-1]["corrections"]:
+                        job.setup()
 
-        if self.gzipped_output:
-            gzip_dir(".")
+                    p = job.run()
+                    # Check for errors using the error handlers and perform
+                    # corrections.
+                    has_error = False
 
-        if self.scratch_dir is not None:
-            for f in os.listdir("."):
-                shutil.copy(f, cwd)
-            shutil.rmtree(tempdir)
-            os.chdir(cwd)
+                    # While the job is running, we use the handlers that are
+                    # monitors to monitor the job.
+                    if isinstance(p, subprocess.Popen):
+                        if self.monitors:
+                            n = 0
+                            while True:
+                                n += 1
+                                time.sleep(self.polling_time_step)
+                                if p.poll() is not None:
+                                    break
+                                if n % self.monitor_freq == 0:
+                                    corrections = _do_check(
+                                        self.monitors,
+                                        terminate_func=p.terminate,
+                                        skip_over_errors=self.skip_over_errors)
+                                    if len(corrections) > 0:
+                                        has_error = True
+                                        total_errors += len(corrections)
+                                        run_log[-1]["corrections"].extend(
+                                            corrections)
+                        else:
+                            p.wait()
 
-        if total_errors >= self.max_errors:
-            logging.info("Max {} errors reached. Exited..."
-                         .format(self.max_errors))
-        elif not unrecoverable:
-            #Cleanup checkpoint files (if any) if run is successful.
-            Custodian._delete_checkpoints(cwd)
+                    # Check for errors again, since in some cases non-monitor
+                    # handlers fix the problems detected by monitors
+                    # if an error has been found, not all handlers need to run
+                    if has_error:
+                        remaining_handlers = filter(lambda x: not x.is_monitor,
+                                                    self.handlers)
+                    else:
+                        remaining_handlers = self.handlers
+
+                    corrections = _do_check(
+                        remaining_handlers,
+                        skip_over_errors=self.skip_over_errors)
+                    if len(corrections) > 0:
+                        has_error = True
+                        total_errors += len(corrections)
+                        run_log[-1]["corrections"].extend(corrections)
+
+                    #Log the corrections to a json file.
+                    with open(Custodian.LOG_FILE, "w") as f:
+                        logging.info("Logging to {}...".format(
+                            Custodian.LOG_FILE))
+                        json.dump(run_log, f, indent=4)
+
+                    # If there are no errors detected, perform
+                    # postprocessing and exit.
+                    if not has_error:
+                        job.postprocess()
+                        break
+                    elif run_log[-1]["corrections"][-1]["actions"] is None:
+                        # Check if there has been an unrecoverable error.
+                        logging.info("Unrecoverable error.")
+                        unrecoverable = True
+                        break
+                    elif total_errors >= self.max_errors:
+                        logging.info("Max errors reached.")
+                        break
+
+                if unrecoverable or total_errors >= self.max_errors:
+                    break
+
+                # Checkpoint after each job so that we can recover from last
+                # point and remove old checkpoints
+                if self.checkpoint:
+                    Custodian._save_checkpoint(cwd, i)
+
+            end = datetime.datetime.now()
+            logging.info("Run ended at {}.".format(end))
+            run_time = end - start
+
+            logging.info("Run completed. Total time taken = {}."
+                         .format(run_time))
+
+            if self.gzipped_output:
+                gzip_dir(".")
+
+            if total_errors >= self.max_errors:
+                logging.info("Max {} errors reached. Exited..."
+                             .format(self.max_errors))
+            elif not unrecoverable:
+                #Cleanup checkpoint files (if any) if run is successful.
+                Custodian._delete_checkpoints(cwd)
 
         return run_log
 
@@ -467,11 +462,10 @@ def backup(filenames, prefix="error"):
                      for f in glob("{}.*.tar.gz".format(prefix))])
     filename = "{}.{}.tar.gz".format(prefix, num + 1)
     logging.info("Backing up run to {}.".format(filename))
-    tar = tarfile.open(filename, "w:gz")
-    for fname in filenames:
-        for f in glob(fname):
-            tar.add(f)
-    tar.close()
+    with tarfile.open(filename, "w:gz") as tar:
+        for fname in filenames:
+            for f in glob(fname):
+                tar.add(f)
 
 
 def gzip_dir(path):
@@ -489,3 +483,67 @@ def gzip_dir(path):
                     GzipFile('{}.gz'.format(f), 'wb') as f_out:
                 f_out.writelines(f_in)
             os.remove(f)
+
+
+def recursive_copy(src, dst):
+    """
+    Implements a recursive copy function similar to Unix's "cp -r" command.
+    Surprisingly, python does not have a real equivalent. shutil.copytree
+    only works if the destination directory is not present.
+
+    Args:
+        src:
+            Source folder to copy.
+        dst:
+            Destination folder.
+    """
+    for parent, subdir, files in os.walk(src):
+        parent = os.path.relpath(parent)
+        realdst = dst if parent == "." else os.path.join(dst, parent)
+        try:
+            os.makedirs(realdst)
+        except Exception as ex:
+            pass
+        for f in files:
+            shutil.copy(os.path.join(parent, f), realdst)
+
+
+class ScratchDir(object):
+    """
+    Creates a with context manager that automatically handles creation of
+    temporary directories in the scratch space and cleanup when done.
+    """
+
+    SCR_LINK = "scratch_link"
+
+    def __init__(self, rootpath):
+        """
+        Args:
+            rootpath:
+                The path in which to create temp subdirectories.
+        """
+        self.rootpath = rootpath
+        self.cwd = os.getcwd()
+
+    def __enter__(self):
+        tempdir = self.cwd
+        if self.rootpath is not None and os.path.abspath(self.rootpath) != \
+                self.cwd:
+            tempdir = tempfile.mkdtemp(dir=self.rootpath)
+            self.tempdir = os.path.abspath(tempdir)
+            recursive_copy(".", tempdir)
+            os.symlink(tempdir, ScratchDir.SCR_LINK)
+            os.chdir(tempdir)
+            logging.info(
+                "Using scratch directory {} and created symbolic "
+                "link called {} in working directory".format(
+                    tempdir, ScratchDir.SCR_LINK))
+        return tempdir
+
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        if self.rootpath is not None and os.path.abspath(self.rootpath) != \
+                self.cwd:
+            recursive_copy(".", self.cwd)
+            shutil.rmtree(self.tempdir)
+            os.chdir(self.cwd)
+            os.remove(ScratchDir.SCR_LINK)
\ No newline at end of file
